
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
The main processing cycle
'''

from zExceptions import NotFound, BadRequest, InternalError, Forbidden, Redirect
from writer import Writer
from actions.base import Action
from actions import *
import re, httplib, datetime, sys
from Products.mechagen.__info__ import __version__, __url__
from datalogger import DataLogger
from Products.mechagen.plugins import get_beautifulsoup_class

class Processor:
    '''Controls the cycle of generation

    '''
    
    def __init__(self):
        self.datalogger = DataLogger()
       
    # --
    # Stub datalogger methods
    # --
    
    def start_logging(self):
        self.datalogger.start_logging()
        
    def stop_logging(self):
        self.datalogger.stop_logging
 
    def log(self, **kw):
        return self.datalogger.log(**kw)

    def log_comment(self, **kw):
        self.datalogger.log_comment(**kw)

    # --
    # Generation support
    # --
       
    def reset_app(self, test):
        'Reset and rebuild the app'
        test.tearDown()
        test.setUp()

    def replay(self, test):
        'Replay the process'
        datalogger = self.datalogger
        # Just to be sure
        datalogger.stop_logging()
        # reset the app first.
        self.reset_app(test)
        # store actions
        self.actions = []
        # iterate on result pages only.
        firstpage = True
        for index, resultpage in enumerate(datalogger):
    
            # handle direct logging, e.g. comments
            if isinstance(resultpage, Action):
                self.actions.append(resultpage)
                continue
                
            # Handle pages that are dealing with an exception.
            if resultpage.response_status is None:
                t, v, tb = resultpage.publish_exc
                assert t is not None
                # first, ignore NotFound because I don't know how to differentiate
                # between a notfound page and a notfound image.
                if t is NotFound or t == 'NotFound':
                    ##print "Skipped / NotFound", index, resultpage.url
                    continue
                # Redirects need no action to be generated; they will be handled from
                # their response.
                if t is Redirect or t == 'Redirect':
                    ##print "Skipped / Redirect", index, resultpage.url
                    continue
                # all the other will pass and get the emission of an ActionFailure
            elif not resultpage.response_is_html:
                # skip non-html pages (except for exceptons handled above)
                # this will filter out all images, css, etc.
                ##print "Skipped", index, resultpage.url, resultpage.response_status
                continue

            # submit page (referer) is the original page to which we submitted to get this result page.
            # (in case of no redirects, it equals to the result page.)
            submitpage_index = datalogger.get_referer_index(index)
            submitpage = datalogger[submitpage_index]
            # prevdata is what submitted to this form. So we have to count back from the submit page.
            for i in range(submitpage_index-1, -1, -1):
                if not isinstance(datalogger[i], Action) and datalogger[i].response_is_html:
                    prevdata = datalogger[i]
                    ##print "This:", index, datalogger[index].url, "Prev:", i, datalogger[i].url
                    break
            else:
                ##print "This:", index, datalogger[index].url, "NOPREV"
                prevdata = None
            
            # On the first page, set the cookies.
            if firstpage and submitpage.cookies:
                self.actions.append(SetCookiesAction(submitpage.cookies))
            action = None

            # see if we could navigate from previous page.
            if prevdata:

                if prevdata.response_forms_error:
                    print 'Internal parse error', prevdata.response_forms_error
                    
                # try to navigate through submit
                # we use referer here as the submit went to that one
                for nr, b_form in enumerate(prevdata.response_forms):
                    if b_form.action == submitpage.url:
                        action = FormSubmitAction(submitpage, prevdata.response_forms, nr)
                        # chech that the action was able to find a control name.
                        if action.submit_control_name is not None:
                            break
                        else:
                            # If not, we continue...
                            action = None
                            # XXX XXX TODO
                            # Really, this is not a nice solution, I should be able to see
                            # this right away, and do not need to make the formdiffs! ...
                        
                if not action:
                    # try to navigate through link clicks
                    for link in prevdata.response_links:
                        # Decode the link's url as it might contain params.
                        # XXX remark: dunno if this is ok. for list, etc... type params
                        querypart = urlencode(submitpage.form.encoded_form) 
                        if querypart:
                            fullurl = '%s?%s' % (submitpage.url, querypart)
                        else:
                            fullurl = submitpage.url
                        if link.url == fullurl:
                            # found a link
                            action = LinkAction(link, prevdata.response_links)
                            break
                        
            if not action:            
                # no match - we open the page as a new url.
                # XXX what happens to ":" conversion and types?
                # form order might change over here...
                action = OpenAction(submitpage, submitpage.form.encoded_form)

            # append action

            if resultpage.publish_exc[0] is None: 
                # append check action (this time, not on submitpage but on
                # the result page)
                self.actions.append(RequestAndCheckAction(action, resultpage))
            else:
                # append action with error
                self.actions.append(RequestWithFailureAction(action, resultpage.publish_exc))

            # reset firstpage flag 
            firstpage = False

    def generate(self, test, buf, methodonly=False,
            oldstyle=False,
            methodname='test_generated',
            browservar = 'b',   # browser var used
            ):
        """Generate the code
        
        If oldstyle is given, my ClientForm extensions are not utilized
        (otherwise, I think, new style is more readible)
        """

        if not isinstance(buf, Writer):
            writer = Writer(buf)
        else:
            writer = buf
    
        # set writer attrs, they are like global vars to be accessed during generation
        writer.oldstyle = oldstyle
        writer.localvars = []
        writer.root_url = test.app.absolute_url()
        try:
            portal = test.portal
        except AttributeError:
            writer.re_path_within_portal = None
        else:
            re_path_within_portal = re.escape(httplib.urlsplit(portal.absolute_url())[2]) + r'(.*)$'
            writer.re_path_within_portal = re.compile(re_path_within_portal)
        writer.action_nr = 0
        writer.browservar = browservar
        
        if not methodonly:
            writer.indenton()
        writer.write('def %s(self):' % (methodname, ))
        writer.newline()
        writer.indenton()
        if self.actions and isinstance(self.actions[0], CommentAction):
            try:
                docstring = self.actions[0].lines[0]
            except IndexError:
                pass
            else:
                writer.write('"""')
                writer.write(docstring)
                writer.newline()
                writer.write('"""')
                writer.newline()
        writer.write('# Generation date: %s' % (str(datetime.datetime.now())[:19], ))
        writer.newline()
        writer.write('# Generated by the "mechagen" utility version %s.' % (__version__, ))
        writer.newline()
        writer.write('# Homepage: %s' % (__url__, ))
        writer.newline()
        writer.newline()
        # code to setup imports
        writer.write('# Standard setup section / imports')
        writer.newline()
        writer.has_beautifulsoup = get_beautifulsoup_class() is not None
        writer.has_plugins = writer.has_beautifulsoup
        writer.use_extended_browser = not oldstyle 
        if writer.use_extended_browser:
            # needs extended browser
            writer.write('from Products.mechagen.runtime import ExtendedBrowser')
            writer.newline()
        else:
            # no runtime support needed
            writer.write('from mechanize import Browser')
            writer.newline()
        if writer.has_plugins:
            # set up beautiful soup class
            writer.write('from Products.mechagen.plugins import get_beautifulsoup_class')
            writer.newline()
        elif writer.has_beautifulsoup:
            # no beautiful soup support
            writer.write('*** WARNING: plugins are inactive, beautiful soup not installed?')
            writer.newline()
        writer.write('from ClientCookie import Cookie')
        writer.newline()
        writer.write('from urllib import urlencode')
        writer.newline()
        writer.write('from urllib2 import URLError, HTTPError')
        writer.newline()
        writer.write('import sys')
        writer.newline()
        writer.write('import base64')
        writer.newline()
        writer.write('import httplib')
        writer.newline()
        writer.write('from cStringIO import StringIO')
        writer.newline()
        # code to setup local vars
        writer.write('# local variable creation')
        writer.newline()
        if writer.use_extended_browser:
            # needs extended browser
            writer.write('%s = ExtendedBrowser()' % (writer.browservar, ))
            writer.newline()
        else:
            # no runtime support needed
            writer.write('%s = Browser()' % (writer.browservar, ))
            writer.newline()
        if writer.has_plugins:
            # set up beautiful soup class
            writer.write('PluginSoup = get_beautifulsoup_class()')
            writer.newline()
        writer.write('root_url = self.app.absolute_url()')
        writer.newline()
        if writer.re_path_within_portal is not None:
            writer.write('portal_url = self.portal.absolute_url()')
            writer.newline()
        writer.newline()
        # generate the test code
        for action in self.actions:
            action.generate(writer)
        
    def check_generated(self, test, text):
        'Check the generated code. Returns exception info.'
        # just to be sure
        self.datalogger.stop_logging()
        # reset the app first.
        self.reset_app(test)
        # compile the code
        # (we need a newline in the end because if we get a comment
        # on the last line, a syntax error would result without)
        code = compile(text + '\n', '<string>', 'exec')
        # execute it: this is kinda dangerous, but ok.
        exec(code)
        try:
            test_generated(test)
        except:
            return sys.exc_info()
        else:
            return (None, None, None)
            
request_processor = Processor()
