
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
General purpose utilities
'''

class counter(dict):
    'Simple counting of keys'

    def get_count(self, key):
        try:
            v = self[key]
        except KeyError:
            v = 0
        return v
    
    def set_count(self, key):
        try:
            v = self[key]
        except KeyError:
            v = 0
        self[key] = v + 1

def lazy(genmethod, number=1):
    '''Decorator for lazy property generation

    e.g.
    
        def _gen(self):
            ...
            return value

        field = lazy(_gen)

   e.g.
    
        def _gen2(self):
            ...
            return value2, value2

        field1, field2 = lazy(_gen2, 2)
    '''

    master_decor = _lazy(genmethod, number)
    if number > 1:
        decors = [master_decor]
        for i in range(1, number):
            decors.append(_lazy(genmethod, number, keyid=master_decor.keyid, index=i))
        return decors
    else:
        return master_decor

class _lazy(object):
    
    def __init__(self, genmethod, maxindex, keyid=None, index=0):
        self.genmethod = genmethod
        self.maxindex = maxindex
        if keyid is None:
            keyid = id(self)
        self.keyid = keyid
        self.index = index

    def __get__(self, obj, cls):
        try:
            clsdict = obj._lazy_clsdict
        except AttributeError:
            clsdict = obj._lazy_clsdict = {}
        try:
            retval = clsdict[self.keyid]
        except KeyError:
            retval = clsdict[self.keyid] = self.genmethod(obj)
        if self.maxindex > 1:
            return retval[self.index]
        else:
            return retval
            
    def __set__(self, obj, value):
        raise Exception, 'Read-only attribute'
        
    def __delete__(self, obj):
        raise Exception, 'Read-only attribute'
