
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Patching the publishing machinery
'''

from processor import request_processor
from forms import Forms
import sys

def patch_zpublisher():
    '''Patch zpublisher to steal logs
    
    This is how we do it.

    The requests go through publish. From publish, processInput is called.
    We record the request after processInput. Then when we get out of
    publish, we inject the response data into a same log record.

    Exceptions raised within publish are considered separately. Now
    there is 'Unauthorized' which we will handle much like a redirect, and
    zExceptions.NotFound which will make a 404. Actually it would be
    better to catch the real response status here but at the moment 
    don't know how to do that.
    '''
    
    # --
    # Patch to steal request
    # --

    from ZPublisher.HTTPRequest import HTTPRequest

    if not hasattr(HTTPRequest, '_patched_processInputs'):

        def processInputs(self, **kw):
            'Substitute method'
            
            HTTPRequest._patched_processInputs(self, **kw)
            
            # Now. We have *everything* in the request
            # so we can log whatever we want.
            data_entry = request_processor.log(
                environ = dict(self.environ),
                cookies = dict(self.cookies),
                form = Forms(self.form),
                )
            # record the logged entry so that we can push
            # the response on it (in case there is one)
            self._logged_data_entry = data_entry
        
            if data_entry:
                # Do not use compression, if I can ask.
                self.set('HTTP_ACCEPT_ENCODING','')
                # without this the response.getBody() could
                # get gzipped, and well... this is an
                # easy solution.

        # Patch the request class.
        HTTPRequest._patched_processInputs,  HTTPRequest.processInputs = \
            HTTPRequest.processInputs, processInputs

    # --
    # Patch to steal response
    # --

    from ZPublisher import Publish
       
    if not hasattr(Publish, '_patched_publish'):

        # substitute method 
        def publish(request, *arg, **kw):

            try:
                response = Publish._patched_publish(request, *arg, **kw)
            except:
                # most importantly, we need to catch 'Unauthorized' here
                # which will be handled differently.
                t, v, tb = sys.exc_info()
                ##print "***", repr(t), repr(v)[:20]
                data_entry = request._logged_data_entry
                if data_entry is not None:
                    # logging is on
                    data_entry.log(
                        response_status = None,
                        response_cookies = {},
                        response_headers = {},
                        response_body = '',
                        publish_exc = (t, v, tb),
                        )
                raise

            # store extra things to the same request we are part of
            data_entry = request._logged_data_entry
            if data_entry is not None:
                # logging is on
                data_entry.log(
                    response_status = response.status,
                    response_cookies = dict(response.cookies),
                    response_headers = dict(response.headers),
                    response_body = str(response.getBody()),
                    publish_exc = (None, None, None)
                    )
           
            return response

        # Patch the response module.
        Publish._patched_publish,  Publish.publish = \
            Publish.publish, publish
