
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Checks the archetypes field error messages
'''

import re

class ATFieldMessages(object):

    class BeautifulSoupMixin:
        
        def at_fieldmessage(self, fieldname):
            'Gets the field error for a tag'
            div = self.first('div', {'id': 'archetypes-fieldname-%s' % fieldname})
            if div['class'].startswith('field error'):
                # there is an error
                return div.fetch('div')[1].string.strip()
            else:
                # no field error
                return None
                
        r_fnid = re.compile(r'archetypes-fieldname-(.*)')
        r_fnclass = re.compile(r'field error(.*)')

        def at_fieldmessages(self):
            'Gets all (at) field errors in a dict'
            result = {}
            for div in self.fetch('div', {'id': self.r_fnid, 'class': self.r_fnclass}):
                fieldname = self.r_fnid.match(div['id']).group(1)
                result[fieldname] = div.fetch('div')[1].string.strip()
            return result

        def at_field(self, fieldname):
            'Gets the at field'
            return self.first('div', {'id': 'archetypes-fieldname-%s' % fieldname})
            
        def at_fields(self):
            'Gets all (at) field errors in a dict'
            result = {}
            for div in self.fetch('div', {'id': self.r_fnid}):
                fieldname = self.r_fnid.match(div['id']).group(1)
                result[fieldname] = div
            return result

    def __init__(self, soup):
        self.fieldnames = soup.at_fields().keys()
        self.field_messages = soup.at_fieldmessages()

    def active(self):
        'Check if needs to be generated'
        # if there are any field messages, we generate for all of them
        return bool(self.field_messages)

    def generate(self, writer):
        writer.write('# Checking Archetypes field error messages')
        writer.newline()
        for fieldname in self.fieldnames:
            try:
                message = self.field_messages[fieldname]
            except KeyError: 
                writer.write('self.assertEqual(soup.at_fieldmessage(%s), None)' % (repr(fieldname), ))
                writer.newline()
            else:
                writer.write('self.assertEqual(soup.at_fieldmessage(%s), %s)' % (repr(fieldname), repr(message)))
                writer.newline()
