
from Testing import ZopeTestCase
from Products.mechagen.generator import MechagenSupport
from Products.mechagen.generator.processor import request_processor
from Products.mechagen.generator.main import Params

class test_method_wrapper(object):

    def __init__(self, testcase, test_method):
        self.testcase = testcase
        self.test_method = test_method

    def __call__(self, *args, **kw):
        # Instead of just executing the test method, run the generation
        # cycle on it.
        #
        # This is done by calling _mechagen_mech with the necessary hooks
        # and parameters.

        gen_method = self.test_method
        
        def end_method(success):
            return False, False
    
        save_method = None  # not important

        def err_method(t, v, tb, params):
            # re-raise same exception
            raise
            
        params = Params(quiet=True)
        params._gen_method_hook = gen_method
        params._end_method_hook = end_method
        params._save_method_hook = save_method
        params._err_method_hook = err_method

        self.testcase.mechagen_single_session(params)
        
class MetaTestCaseMixin(MechagenSupport, ZopeTestCase.Functional):
    '''Meta test case

    Use this as a mixin to your required test case to test
    mechagen functionality itself.

    All the test methods in the case will be run on a sandbox, autogenerated
    and the results replayed. The test will pass only if all this went well.
    
    Basically, all the test methods that you build up this way will run
    through a complete generation cycle. This way you make test against
    the mechagen core.
    '''
    
    def __call__(self, result=None):
        # Wrap the test method. But only if we are not calling via mechagen.
        # This allows us to use mechagen itself to these tests (to autogenerate)
        if self.mechagen_env_var() is None:
            # Wrap the test method.
            setattr(self, self._TestCase__testMethodName, 
                test_method_wrapper(self, getattr(self, self._TestCase__testMethodName)))

        # Really call it. (Suppose noone is overriding TestCase.__call__, also
        # suppose we are finally inheriting from at least the base.TestCase of Zope.)
        ZopeTestCase.TestCase.__call__(self, result=result)
